
/*******************************************************************************
*
* File check1.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Drawing of branch diagrams.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "plot.h"

static int *s1=NULL,*s2=NULL;


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check1.c]","Unknown theory");

   return THEORIES;
}


static int select1(theory_t th,int order,int nbr)
{
   int nvm,ibr,ns1,n;
   vertex_t *v1,*v2;
   branch_t *br;
   vlist_t vl;

   if (s1!=NULL)
      free(s1);

   s1=malloc(nbr*sizeof(*s1));
   error(s1==NULL,1,"select1 [check1.c]","Unable to allocate auxiliary array");

   for (ibr=0;ibr<nbr;ibr++)
      s1[ibr]=0;

   nvm=order+2;
   br=alloc_branch(nvm);
   vl=vertices(th,order);
   ns1=0;

   for (ibr=0;ibr<nbr;ibr++)
   {
      (*br).nv=nvm;
      get_branch(th,ibr,br);

      n=(*br).nv-1;
      v1=vl.v[(*br).v[0][0]]+(*br).v[0][1];
      v2=vl.v[(*br).v[n][0]]+(*br).v[n][1];

      if (((*v1).type!=SOURCE)&&((*v2).type!=SOURCE))
      {
         if (ibr<=inverse_branch(th,ibr))
         {
            s1[ibr]=1;
            ns1+=1;
         }
      }
   }

   free_branch(br);

   return ns1;
}


static int select2(theory_t th,int order,int nbr)
{
   int nvm,ibr,ns2,n,i;
   vertex_t *v1,*v2;
   branch_t *br;
   vlist_t vl;

   if (s2!=NULL)
      free(s2);

   s2=malloc(nbr*sizeof(*s2));
   error(s2==NULL,1,"select2 [check1.c]","Unable to allocate auxiliary array");

   for (ibr=0;ibr<nbr;ibr++)
      s2[ibr]=0;

   nvm=order+2;
   br=alloc_branch(nvm);
   vl=vertices(th,order);
   ns2=0;

   for (ibr=0;ibr<nbr;ibr++)
   {
      if (s1[ibr]==1)
      {
         (*br).nv=nvm;
         get_branch(th,ibr,br);

         n=(*br).nv-1;
         v1=vl.v[(*br).v[0][0]]+(*br).v[0][1];
         v2=vl.v[(*br).v[n][0]]+(*br).v[n][1];

         if ((*v1).type==ACTION)
         {
            n=0;

            for (i=0;i<(int)(LTYPES);i++)
               n+=(*v1).nl[i];

            if ((n==3)&&(((*br).nv==1)||((*br).l[0]==GLUON))&&
                (((*v1).nl[(int)(GHOST)]>0)||((*v1).nl[(int)(QUARK)]>0)))
               continue;
         }

         if ((*v2).type==ACTION)
         {
            n=0;

            for (i=0;i<(int)(LTYPES);i++)
               n+=(*v2).nl[i];

            if ((n==3)&&(((*br).nv==1)||((*br).l[(*br).nv-2]==GLUON))&&
                (((*v2).nl[(int)(GHOST)]>0)||((*v2).nl[(int)(QUARK)]>0)))
               continue;
         }

         s2[ibr]=1;
         ns2+=1;
      }
   }

   return ns2;
}


int main(void)
{
   int order,nbr,ns1,ns2,nvm,ip,io,ibr;
   double pos1[2],pos2[2];
   char name[5],plot_file[NAME_SIZE];
   theory_t th;
   branch_t *br;
   vlist_t vl;
   FILE *plt;

   printf("\n");
   printf("Drawing of branches\n");
   printf("-------------------\n\n");

   while (1)
   {
      printf("Specify theory and maximal order: ");
      (void)scanf("%4s %d",name,&order);

      th=theory(name);
      nbr=branches(th,order);
      printf("\n");
      printf("Total number of branches = %d\n",nbr);

      ns1=select1(th,order,nbr);
      printf("Number of branches w/o SOURCE vertices = %d\n\n",ns1);

      printf("Print these branches? (1|0) ");
      (void)scanf("%d",&ip);

      if (ip==1)
      {
         sprintf(plot_file,"plots/%s.%d.s1.tex",name,order);
         plt=fopen(plot_file,"w");
         error(plt==NULL,1,"main [check1.c]","Unable to open plot file");

         nvm=order+2;
         br=alloc_branch(nvm);
         vl=vertices(th,order);
         start_plot(plt,3);

         for (io=0;io<=order;io++)
         {
            for (ibr=0;ibr<nbr;ibr++)
            {
               if ((s1[ibr]!=0)&&(io==get_order(th,ibr)))
               {
                  (*br).nv=nvm;
                  get_branch(th,ibr,br);

                  start_bbox();
                  draw_branch(br,&vl);
                  end_bbox(pos1,pos2);

                  start_diagram(pos1,pos2);
                  draw_branch(br,&vl);
                  end_diagram();
               }
            }
         }

         end_plot();
         fclose(plt);
         printf("Printed drawing to file %s\n\n",plot_file);
      }

      if (th!=PHI4)
      {

         ns2=select2(th,order,nbr);
         printf("After removal of diagrams that contract to 0 = %d\n\n",ns2);

         printf("Print these branches? (1|0) ");
         (void)scanf("%d",&ip);

         if (ip==1)
         {
            sprintf(plot_file,"plots/%s.%d.s2.tex",name,order);
            plt=fopen(plot_file,"w");
            error(plt==NULL,1,"main [check1.c]","Unable to open plot file");

            nvm=order+2;
            br=alloc_branch(nvm);
            vl=vertices(th,order);
            start_plot(plt,3);

            for (io=0;io<=order;io++)
            {
               for (ibr=0;ibr<nbr;ibr++)
               {
                  if ((s2[ibr]!=0)&&(io==get_order(th,ibr)))
                  {
                     (*br).nv=nvm;
                     get_branch(th,ibr,br);

                     start_bbox();
                     draw_branch(br,&vl);
                     end_bbox(pos1,pos2);

                     start_diagram(pos1,pos2);
                     draw_branch(br,&vl);
                     end_diagram();
                  }
               }
            }

            end_plot();
            fclose(plt);
            printf("Printed drawing to file %s\n\n",plot_file);
         }
      }
   }

   exit(0);
}
