
/*******************************************************************************
*
* File check2.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Drawing of tree diagrams.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "rvt.h"
#include "plot.h"


static int nvs=0;
static ltype_t **imat;
static vlist_t vls;


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check2.c]","Unknown theory");

   return THEORIES;
}


static void alloc_imat(int nv)
{
   int i;
   ltype_t **pp,*p;

   if (nv>nvs)
   {
      if (nvs>0)
      {
         free(imat[0]);
         free(imat);
      }

      nvs=nv;
      pp=malloc(nv*sizeof(*pp));
      p=malloc(nv*nv*sizeof(*p));

      error((pp==NULL)||(p==NULL),1,"alloc_imat [check2.c]",
            "Unable to allocate auxiliary array");

      imat=pp;

      for (i=0;i<nv;i++)
      {
         imat[i]=p;
         p+=nv;
      }
   }
}


static void set_imat(diagram_t *dm)
{
   int nv,nl,i,j,k;
   line_t *l;

   nv=(*dm).nv;
   nl=(*dm).nl;
   l=(*dm).l;

   for (i=0;i<nv;i++)
   {
      for (j=0;j<nv;j++)
         imat[i][j]=LTYPES;
   }

   for (k=0;k<nl;k++)
   {
      i=(*l).iv[0];
      j=(*l).iv[1];

      imat[i][j]=(*l).type;
      imat[j][i]=rvt[(int)((*l).type)];

      l+=1;
   }
}


static int chk_gfix(diagram_t *dm)
{
   int nv,i;
   int (*v)[2];
   vertex_t *vx;

   nv=(*dm).nv;
   v=(*dm).v;

   for (i=0;i<nv;i++)
   {
      vx=vls.v[v[i][0]]+v[i][1];

      if ((*vx).type==GFIX)
         return 1;
   }

   return 0;
}


static int chk_vff(diagram_t *dm)
{
   int nv,n,i,j,k;
   int (*v)[2];
   vertex_t *vx;

   error(is_tree(dm)!=1,1,"chk_vff [check2.c]",
         "Diagram is not a tree diagram");

   set_imat(dm);
   nv=(*dm).nv;
   v=(*dm).v;

   for (i=0;i<nv;i++)
   {
      n=0;
      k=nv;

      for (j=0;j<nv;j++)
      {
         if (imat[i][j]!=LTYPES)
         {
            k=j;
            n+=1;
         }
      }

      if ((nv==1)||((n==1)&&(imat[i][k]==GLUON)))
      {
         vx=vls.v[v[i][0]]+v[i][1];
         n=0;

         for (j=0;j<(int)(LTYPES);j++)
            n+=(*vx).nl[j];

         if ((n==3)&&
             (((*vx).nl[(int)(GHOST)]>0)||((*vx).nl[(int)(QUARK)]>0)))
            return 1;
      }
   }

   return 0;
}


static dlist_t *select1(dlist_t *dl)
{
   int n,i;
   diagram_t **dma;
   dlist_t *dln;

   n=(*dl).n;
   dma=(*dl).dma;
   dln=alloc_dlist(n,0);

   for (i=0;i<n;i++)
   {
      if ((chk_gfix(dma[i])==0)&&(chk_vff(dma[i])==0))
         add_dgm(dma[i],dln);
   }

   return dln;
}


int main(void)
{
   int order,nbr,io,ib,ip;
   int nc,n,i;
   double pos1[2],pos2[2];
   char name[5],plot_file[NAME_SIZE];
   theory_t th;
   diagram_t **dma;
   dlist_t *dl,*dlw;
   vlist_t vl;
   FILE *plt;

   printf("\n");
   printf("Drawing of tree diagrams\n");
   printf("------------------------\n\n");

   while (1)
   {
      printf("Specify theory and maximal order: ");
      (void)scanf("%4s %d",name,&order);

      th=theory(name);
      trees(th,order);
      vls=vertices(th,order);
      alloc_imat(order+2);

      printf("Number of tree diagrams:\n");
      printf("  order   nbranch   ntree\n");

      for (io=0;io<=order;io++)
      {
         nbr=nbr_max(th,io);

         for (ib=1;ib<=nbr;ib++)
         {
            dl=select_trees(th,io,ib);

            if (dl!=NULL)
               printf("    %d        %d     %5d\n",io,ib,(*dl).n);
            else
               printf("    %d        %d     %5d\n",io,ib,0);
         }
      }

      printf("\n");
      printf("Print these diagrams? (1|0) ");
      (void)scanf("%d",&ip);

      if (ip==1)
      {
         printf("Number of columns = ");
         (void)scanf("%d",&nc);
         if (nc<1)
            nc=1;

         sprintf(plot_file,"plots/%s.%d.s3.tex",name,order);
         plt=fopen(plot_file,"w");
         error(plt==NULL,1,"main [check3.c]","Unable to open plot file");
         start_plot(plt,nc);
         vl=vertices(th,order);

         for (io=0;io<=order;io++)
         {
            nbr=nbr_max(th,io);

            for (ib=1;ib<=nbr;ib++)
            {
               dl=select_trees(th,io,ib);

               if (dl!=NULL)
               {
                  n=(*dl).n;
                  dma=(*dl).dma;

                  for (i=0;i<n;i++)
                  {
                     start_bbox();
                     draw_tree(dma[i],&vl);
                     end_bbox(pos1,pos2);

                     start_diagram(pos1,pos2);
                     draw_tree(dma[i],&vl);
                     end_diagram();
                  }
               }
            }
         }

         end_plot();
         fclose(plt);
         printf("Printed plot commands to file %s\n\n",plot_file);
      }

      if (th!=PHI4)
      {
         printf("Number of tree diagrams w/o GFIX and fermion V's:\n");
         printf("  order   nbranch   ntree\n");

         for (io=0;io<=order;io++)
         {
            nbr=nbr_max(th,io);

            for (ib=1;ib<=nbr;ib++)
            {
               dl=select_trees(th,io,ib);

               if (dl!=NULL)
               {
                  dlw=select1(dl);
                  printf("    %d        %d     %5d\n",io,ib,(*dlw).n);
                  free_dlist(dlw);
               }
            }
         }

         printf("\n");
         printf("Print these diagrams? (1|0) ");
         (void)scanf("%d",&ip);

         if (ip==1)
         {
            printf("Number of columns = ");
            (void)scanf("%d",&nc);
            if (nc<1)
               nc=1;

            sprintf(plot_file,"plots/%s.%d.s4.tex",name,order);
            plt=fopen(plot_file,"w");
            error(plt==NULL,1,"main [check3.c]","Unable to open plot file");
            start_plot(plt,2);
            vl=vertices(th,order);

            for (io=0;io<=order;io++)
            {
               nbr=nbr_max(th,io);

               for (ib=1;ib<=nbr;ib++)
               {
                  dl=select_trees(th,io,ib);

                  if (dl!=NULL)
                  {
                     dlw=select1(dl);

                     n=(*dlw).n;
                     dma=(*dlw).dma;

                     for (i=0;i<n;i++)
                     {
                        start_bbox();
                        draw_tree(dma[i],&vl);
                        end_bbox(pos1,pos2);

                        start_diagram(pos1,pos2);
                        draw_tree(dma[i],&vl);
                        end_diagram();
                     }

                     free_dlist(dlw);
                  }
               }
            }

            end_plot();
            fclose(plt);
            printf("Printed plot commands to file %s\n\n",plot_file);
         }
      }
   }

   exit(0);
}
