
/*******************************************************************************
*
* File check3.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Generation of labeled tree diagrams.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "trees.h"

static vlist_t vls;


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check3.c]","Unknown theory");

   return THEORIES;
}


static void check_tlist(theory_t th,int order,tlist_t *tl)
{
   int nva,nla,nv,nl;
   int n,i,ie;
   diagram_t *dmw;
   ltree_t **lta;

   n=(*tl).n;
   lta=(*tl).lta;
   nva=0;
   nla=0;

   for (i=0;i<n;i++)
   {
      nv=lta[i][0].dm[0].nv;
      nl=lta[i][0].dm[0].nl+lta[i][0].nl;

      if (nv>nva)
         nva=nv;
      if (nl>nla)
         nla=nl;
   }

   dmw=alloc_dgm(nva,nla);
   ie=0;

   for (i=0;i<n;i++)
   {
      (*dmw).nv=nva;
      (*dmw).nl=nla;
      ltd2dgm(lta[i],dmw);
      ie|=(check_dgm(th,order,dmw)<2);
   }

   error(ie!=0,1,"check_tlist [check3.c]",
         "There are diagrams with less than 2 external lines");
   free(dmw);
}


static int nogfix(tlist_t *tl)
{
   int n,i,nd;
   ltree_t **lta;

   n=(*tl).n;
   lta=(*tl).lta;
   nd=0;

   for (i=0;i<n;i++)
   {
      if (has_gfix(lta[i][0].dm,&vls)==0)
         nd+=1;
   }

   return nd;
}


static int noghost(tlist_t *tl)
{
   int n,i,nd;
   ltree_t **lta;

   n=(*tl).n;
   lta=(*tl).lta;
   nd=0;

   for (i=0;i<n;i++)
   {
      if ((has_gfix(lta[i][0].dm,&vls)==0)&&
          (has_ghost(lta[i][0].dm,&vls)==0))
         nd+=1;
   }

   return nd;
}


int main(void)
{
   int order,is,nlp;
   int na0,na1,io,il;
   char name[5];
   theory_t th;
   tlist_t *tl;

   printf("\n");
   printf("Generation of labeled tree diagrams\n");
   printf("-----------------------------------\n\n");

   while (1)
   {
      printf("Specify theory, maximal order and root selection flag: ");
      (void)(scanf("%4s %d %d",name,&order,&is));

      th=theory(name);
      na0=trees(th,order);
      na1=ltrees(th,order);
      set_root(th,is);

      printf("\n");
      printf("Total number of basic tree diagrams =   %d\n",na0);
      printf("Total number of labeled tree diagrams = %d\n\n",na1);

      printf("Numbers of tree diagrams of a given order:\n");
      printf("  order    nloop     ntree\n");

      for (io=1;io<=order;io++)
      {
         nlp=nlp_max(th,io);

         for (il=0;il<=nlp;il++)
         {
            tl=select_ltrees(th,io,il);
            check_tlist(th,io,tl);
            printf("    %d        %d     %5d\n",
                   io,il,(*tl).n);
         }
      }

      printf("\n");

      if (th!=PHI4)
      {
         vls=vertices(th,order);

         printf("Numbers of tree diagrams w/o GFIX vertices:\n");
         printf("  order    nloop     ntree\n");

         for (io=1;io<=order;io++)
         {
            nlp=nlp_max(th,io);

            for (il=0;il<=nlp;il++)
            {
               tl=select_ltrees(th,io,il);
               printf("    %d        %d     %5d\n",
                      io,il,nogfix(tl));
            }
         }

         printf("\n");
         printf("Numbers of tree diagrams w/o ghost & GFIX vertices:\n");
         printf("  order    nloop     ntree\n");

         for (io=1;io<=order;io++)
         {
            nlp=nlp_max(th,io);

            for (il=0;il<=nlp;il++)
            {
               tl=select_ltrees(th,io,il);
               printf("    %d        %d     %5d\n",
                      io,il,noghost(tl));
            }
         }
      }

      printf("\n");
   }

   exit(0);
}
